/*************************************************
* BigInt Operators Source File                   *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/bigint.h>
#include <opencl/mp_core.h>

namespace OpenCL {

/*************************************************
* Addition Operator                              *
*************************************************/
BigInt& BigInt::operator+=(const BigInt& n)
   {
   return (*this = *this + n);
   }

/*************************************************
* Subtraction Operator                           *
*************************************************/
BigInt& BigInt::operator-=(const BigInt& n)
   {
   return (*this = *this - n);
   }

/*************************************************
* Multiplication Operator                        *
*************************************************/
BigInt& BigInt::operator*=(const BigInt& n)
   {
   return (*this = *this * n);
   }

/*************************************************
* Division Operator                              *
*************************************************/
BigInt& BigInt::operator/=(const BigInt& n)
   {
   return (*this = *this / n);
   }

/*************************************************
* Modulo Operator                                *
*************************************************/
BigInt& BigInt::operator%=(const BigInt& mod)
   {
   return (*this = *this % mod);
   }

/*************************************************
* Modulo Operator                                *
*************************************************/
u32bit BigInt::operator%=(u32bit mod)
   {
   *this = *this % mod;
   return at(0);
   }

/*************************************************
* Left Shift Operator                            *
*************************************************/
BigInt& BigInt::operator<<=(u32bit shift)
   {
   if(shift == 0) return *this;
   const u32bit shift_words = shift / 32, shift_bits  = shift % 32;

   grow_to(sig_words() + shift_words + (shift_bits ? 1 : 0));
   bigint_shl1(get_reg(), sig_words(), shift_words, shift_bits);
   return *this;
   }

/*************************************************
* Right Shift Operator                           *
*************************************************/
BigInt& BigInt::operator>>=(u32bit shift)
   {
   if(shift == 0) return *this;

   if(bits() <= shift)
      {
      *this = zero();
      return *this;
      }

   const u32bit shift_words = shift / 32, shift_bits  = shift % 32;
   bigint_shr1(get_reg(), sig_words(), shift_words, shift_bits);
   return *this;
   }

}
