/*************************************************
* Three Operand MP Algorithms Source File        *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/mp_core.h>

extern "C" {

/*************************************************
* Three Operand Addition                         *
*************************************************/
void bigint_add3(u32bit* z,
                 const u32bit* x, u32bit x_size,
                 const u32bit* y, u32bit y_size)
   {
   if(x_size < y_size) { bigint_add3(z, y, y_size, x, x_size); return; }

   u32bit carry = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      u64bit sum = ((u64bit)x[j] + y[j]) + carry;
      z[j] = (sum & 0xFFFFFFFF);
      carry = (sum >> 32) & 0xFFFFFFFF;
      }
   for(u32bit j = y_size; j != x_size; j++)
      {
      u64bit sum = (u64bit)x[j] + carry;
      z[j] = (sum & 0xFFFFFFFF);
      carry = (sum >> 32) & 0xFFFFFFFF;
      }
   z[x_size] = carry;
   }

/*************************************************
* Three Operand Subtraction                      *
*************************************************/
void bigint_sub3(u32bit* z,
                 const u32bit* x, u32bit x_size,
                 const u32bit* y, u32bit y_size)
   {
   static const u64bit OFFSET = 0x100000000;
   u32bit borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      u64bit temp = ((OFFSET + x[j]) - y[j]) - borrow;
      borrow = ((temp < OFFSET) ? 1 : 0);
      z[j] = (u32bit)(temp - OFFSET);
      }
   for(u32bit j = y_size; j != x_size; j++)
      {
      u64bit temp = (OFFSET + x[j]) - borrow;
      borrow = ((temp < OFFSET) ? 1 : 0);
      z[j] = (u32bit)(temp - OFFSET);
      }
   }

/*************************************************
* Three Operand Multiplication                   *
*************************************************/
void bigint_mul3(u32bit* z,
                 const u32bit* x, u32bit x_size,
                 const u32bit* y, u32bit y_size)
   {
   for(u32bit j = 0; j != x_size; j++)
      {
      u32bit carry = 0;
      for(u32bit k = 0; k != y_size; k++)
         {
         u64bit product = (u64bit)z[j+k] + (u64bit)x[j] * y[k] + carry;
         z[j+k] = (product & 0xFFFFFFFF);
         carry = (product >> 32) & 0xFFFFFFFF;
         }
      z[j+y_size] = carry;
      }
   }

/*************************************************
* Three Operand Linear Multiply                  *
*************************************************/
void bigint_linmul3(u32bit* z, const u32bit* x, u32bit x_size, u32bit y)
   {
   for(u32bit j = 0; j != x_size; j++)
      {
      u32bit carry = 0;
      u64bit product = (u64bit)z[j] + (u64bit)x[j] * y + carry;
      z[j] = (product & 0xFFFFFFFF);
      carry = (product >> 32) & 0xFFFFFFFF;
      z[j+1] = carry;
      }
   }

}
